<!-- 
PluginName: Icon Filter
PluginUrl: https://ioctopus.online/icon-filter
Version: 1.8
Propose to set up: true

-->




<script>
{

  // ########  ##     ## ######## ########  #######  ##    ##
  // ##     ## ##     ##    ##       ##    ##     ## ###   ##
  // ##     ## ##     ##    ##       ##    ##     ## ####  ##
  // ########  ##     ##    ##       ##    ##     ## ## ## ##
  // ##     ## ##     ##    ##       ##    ##     ## ##  ####
  // ##     ## ##     ##    ##       ##    ##     ## ##   ###
  // ########   #######     ##       ##     #######  ##    ##
  const iconButton = {

    components: {
      buttonus: ioctopus.getButtonusComponent(),
    },


    props: {
      iconKey: String,
      toggle: Boolean,
      icon: String,
    },


    data() {

      return {
        moving: {
          startEvent: null,
          startButtonX: null,
          lastEvent: null,
          moved: false,
          length: 0,
          startGlobalPos: { x: 0 },
          pos: { x: 0, y: 0 },
          process: false,
        },
        containerSize: { x: 0, y: 0 },
      }
    },


    computed: {

      containerStyles() {
        const styles = {}

        if (this.containerSize.x) {
          styles.width = this.containerSize.x + 'px'
        }

        if (this.containerSize.y) {
          styles.height = this.containerSize.y + 'px'
        }

        return styles
      },

    },


    methods: {

      movingStart(e) {
        this.moving.startEvent = e
        this.moving.lastEvent = e
        this.moving.length = 0
        this.moving.process = true

        this.containerSize.x = this.$el.clientWidth
        this.containerSize.y = this.$el.clientHeight

        const bounds = this.$el.getBoundingClientRect()
        this.moving.startButtonX = e.pageX - bounds.x
        this.moving.startGlobalPos.x = bounds.left

        const onMove = ($event) => {
          this.onMove($event)
        }

        const onMoveStop = ($event) => {
          document.removeEventListener('mousemove', onMove)
          document.removeEventListener('mouseup', onMoveStop)
          this.onMoveStop($event)
        }

        document.addEventListener('mousemove', onMove)
        document.addEventListener('mouseup', onMoveStop)
      },


      onMove(e) {

        // this.moving.pos.x = (e.pageX - this.moving.startEvent.pageX)

        const delta = {
          x: e.pageX - this.moving.lastEvent.pageX,
        }

        this.moving.length += Math.abs(delta.x)

        this.moving.lastEvent = e
        this.moveButtonToMouse()

        this.$emit('move')
      },


      onMoveStop() {
        this.moving.process = false
        this.moving.startEvent = null
        this.moving.lastEvent = null
        this.containerSize.x = 0
        this.containerSize.y = 0
        this.moving.pos.x = 0
        this.$emit('move-stop')
      },


      toggleEmit($event) {
        if (this.moving.length > 5) return
        this.$emit('update:toggle', $event)
      },


      moveButtonToMouse() {
        const bounds = this.$el.getBoundingClientRect()
        let currentDelta = this.moving.lastEvent.pageX - bounds.x
        let delta = currentDelta - this.moving.startButtonX
        this.moving.pos.x = delta
      },

    },


    template: `
    <div
      :class="['icon-filter--button-container', {
        'icon-filter--button-container-process': moving.process,
      }]"
      :style="containerStyles"
    >
      <buttonus
        :style="{
          left: moving.pos.x+'px',
        }"
        :icon="icon"
        :toggle="toggle"
        @mousedown="movingStart"
        @update:toggle="toggleEmit"
      />
    </div>
    `
  }



  // ##      ## #### ########   ######   ######## ########
  // ##  ##  ##  ##  ##     ## ##    ##  ##          ##
  // ##  ##  ##  ##  ##     ## ##        ##          ##
  // ##  ##  ##  ##  ##     ## ##   #### ######      ##
  // ##  ##  ##  ##  ##     ## ##    ##  ##          ##
  // ##  ##  ##  ##  ##     ## ##    ##  ##          ##
  //  ###  ###  #### ########   ######   ########    ##
  let widget = new Vue({

    components: {
      buttonus: ioctopus.getButtonusComponent(),
      iconButton,
    },


    data() {
      ioctopus.$store.getters.mapData.plugins.iconFilter = ioctopus.$store.getters.mapData.plugins.iconFilter || {
        sorting: {},
      }

      return {
        iconKeys: [],
        selected: {
          '': false,
        },
        _init: false,
        buttonClicked: false,
        sorting: ioctopus.$store.getters.mapData.plugins.iconFilter.sorting,
      }
    },


    //  ######   #######  ##     ## ########  ##     ## ######## ######## ########
    // ##    ## ##     ## ###   ### ##     ## ##     ##    ##    ##       ##     ##
    // ##       ##     ## #### #### ##     ## ##     ##    ##    ##       ##     ##
    // ##       ##     ## ## ### ## ########  ##     ##    ##    ######   ##     ##
    // ##       ##     ## ##     ## ##        ##     ##    ##    ##       ##     ##
    // ##    ## ##     ## ##     ## ##        ##     ##    ##    ##       ##     ##
    //  ######   #######  ##     ## ##         #######     ##    ######## ########
    computed: {

        clearButtonVisible() {
          for (let key in this.selected) {
            if (key && this.selected[key]) return true
          }
          return false
        },


        unsortedList() {
          const list = []

          for (let iconKey of this.iconKeys) {
            if (!iconKey) continue

            let sorting = this.sorting[iconKey]
            if (typeof sorting === 'undefined') {
              sorting = 100000
            }

            list.push({
              iconKey,
              sorting,
            })
          }

          return list
        },


        sortedIcons() {
          return this.unsortedList.sort((a, b) => {
            if (a.sorting > b.sorting) return 1
            if (a.sorting < b.sorting) return -1
            return 0
          })
        },

      },


    // ##      ##    ###    ########  ######  ##     ##
    // ##  ##  ##   ## ##      ##    ##    ## ##     ##
    // ##  ##  ##  ##   ##     ##    ##       ##     ##
    // ##  ##  ## ##     ##    ##    ##       #########
    // ##  ##  ## #########    ##    ##       ##     ##
    // ##  ##  ## ##     ##    ##    ##    ## ##     ##
    //  ###  ###  ##     ##    ##     ######  ##     ##
    watch: {

      iconKeys(iconKeys, prevKeys) {
        prevKeys = ioctopus._.cloneDeep(prevKeys)

        for (let iconKey of iconKeys) {
          if (typeof this.selected[iconKey] === 'undefined') {
            this.$set(this.selected, iconKey, false)
          }
        }

        if (!this._init) {
          this._init = true
          return
        }

        this.updateBranchVisible()
      },


      selected: {
        deep: true,
        handler() {
          this.updateBranchVisible()
        },
      },

    },


    // #    # ###### ##### #    #  ####  #####   ####
    // ##  ## #        #   #    # #    # #    # #
    // # ## # #####    #   ###### #    # #    #  ####
    // #    # #        #   #    # #    # #    #      #
    // #    # #        #   #    # #    # #    # #    #
    // #    # ######   #   #    #  ####  #####   ####
    methods: {

      updateBranchVisible() {
        if (!this.buttonClicked) return

        const visibleKeys = []
        for (let iconKey of this.iconKeys) {
          if (this.selected[iconKey]) {
            visibleKeys.push(iconKey)
          }
        }

        if (visibleKeys && visibleKeys.length) {
          for (let branchId in ioctopus.structureInfo.byId) {

            /** @type {structureInfoItem} */
            const item = ioctopus.structureInfo.byId[branchId]

            // Root
            if (!item.parentId) continue

            /**
             * @type {string[]}
             */
            const branchIconKeys = item.mapData
                && item.mapData.content
                && item.mapData.content.icon
                && item.mapData.content.icon.keys

            const visible = branchIconKeys &&
                branchIconKeys.filter(n => visibleKeys.indexOf(n) !== -1).length

            ioctopus.setBranchVisible(branchId, visible)
          }
        }
        else {
          for (let branchId in ioctopus.structureInfo.byId) {
            ioctopus.setBranchVisible(branchId, false)
          }
        }
      },


      toggle(iconKey, selected) {
        this.buttonClicked = true
        this.$set(this.selected, iconKey, selected)
      },


      clear() {
        this.buttonClicked = true

        for (let key in this.selected) {
          if (this.selected[key]) {
            this.$set(this.selected, key, false)
          }
        }
      },


      sortByX(movingElementKey) {

        // Получаем передвигаемую кнопку
        let movingButton = this.$refs['button' + movingElementKey][0]

        // X всех кнопок
        let xButtons = []

        // Перебираем кнопки
        for (let iconKey of this.iconKeys) {
          const $button = this.$refs['button' + iconKey][0]
          const $el = $button.$el.childNodes[0]
          const bounds = $el.getBoundingClientRect()
          xButtons.push({
            iconKey,
            x: bounds.left,
            w: bounds.width,
          })
        }

        xButtons.sort((a, b) => {
          if (a.x > b.x) return 1
          if (a.x < b.x) return -1
          return 0
        })

        let i = 0
        let sorting = {}
        for (let xButton of xButtons) {
          sorting[xButton.iconKey] = i
          i ++
        }

        // console.log(xButtons.map(i => i.iconKey))

        this.$set(this, 'sorting', sorting)
        movingButton.moveButtonToMouse()
        this.$nextTick(() => {
          movingButton.moveButtonToMouse()
        })
      },


      save() {
        ioctopus.$store.getters.mapData.plugins.iconFilter.sorting = this.sorting
        ioctopus.save()
      },

    },


    template: `
      <div class="icon-filter--container">

        <!-- Icons -->
        <div class="icon-filter--buttons-container" ref="buttons-container">
          <icon-button
            v-for="element in sortedIcons"
            :key="'icon-filter-'+element.iconKey"
            :ref="'button' + element.iconKey"
            :icon="'fas fa-'+element.iconKey"
            :toggle="selected[element.iconKey]"
            :icon-key="element.iconKey"
            @update:toggle="toggle(element.iconKey, $event)"
            @move="sortByX(element.iconKey)"
            @move-stop="save"
          />

        </div>

        <!-- Clear -->
        <buttonus
          icon="fas fa-times"
          :enabled="clearButtonVisible"
          title="Clear filter"
          @click="clear"
        />

      </div>
    `,

  })


  // On MapData
	ioctopus.$on('map-data', mapData => {

    widget.$set(widget, 'iconKeys', [])

    // Обновляем список добавленных иконок
    const add = iconKey => {
      if (widget.iconKeys.indexOf(iconKey) !== -1) return
      widget.iconKeys.push(iconKey)
    }

    const addList = list => {
      if (!list) return

      for (let item of list) {
        if (item.content && item.content.icon && item.content.icon.keys) {
          for (let iconKey of item.content.icon.keys) {
            add(iconKey)
          }
        }

        addList(item.items)
      }
    }

    addList(mapData.items)
  })


  // On platform ready
  ioctopus.addElement({
    name: 'icons-filter',
    instance: widget,
  })
}
</script>



<style>
.icon-filter--container {
  display: flex;
}

.icon-filter--buttons-container {
  display: flex;
}

.icon-filter--button-container {

}

.icon-filter--button-container-process {
  position: relative;
  z-index: 10;
}

.icon-filter--container .buttonus--button:hover {
  background: var(--panel--main--hover--background);
  color: var(--panel--main--hover--color);
}

.icon-filter--container .buttonus--button.buttonus--active {
  background: var(--panel--main--active--background);
  color: var(--panel--main--active--color);
}

.icon-filter--reset {
  position: absolute;
  right: -20px;
  transform: translateX(100%);
  color: var(--panel--main--color);

}

.icon-filter--reset:hover {
  color: var(--panel--main--hover--color);
}

@media (max-width: 768px) {
  .icon-filter--reset {
    display: none;
  }
}
</style>
